from __future__ import annotations

import logging
from collections.abc import Sequence
from functools import partial
from typing import TYPE_CHECKING, Any, Callable, Literal, cast

from huggingface_hub import get_safetensors_metadata
from huggingface_hub.errors import (
    GatedRepoError,
    NotASafetensorsRepoError,
    SafetensorsParsingError,
)
from pydantic import BaseModel, ConfigDict, field_validator

from mteb.abstasks.AbsTask import AbsTask
from mteb.encoder_interface import Encoder

from .custom_validators import LICENSES, MODALITIES, STR_DATE, STR_URL
from .languages import (
    ISO_LANGUAGE_SCRIPT,
    check_language_code,
)

if TYPE_CHECKING:
    from .models.sentence_transformer_wrapper import SentenceTransformerWrapper

logger = logging.getLogger(__name__)


FRAMEWORKS = Literal[
    "Sentence Transformers",
    "PyTorch",
    "GritLM",
    "LLM2Vec",
    "TensorFlow",
    "API",
    "Tevatron",
    "NumPy",
    "PyLate",
    "ColBERT",
]
DISTANCE_METRICS = Literal["cosine", "max_sim", "dot"]


def sentence_transformers_loader(
    model_name: str, revision: str | None = None, **kwargs
) -> SentenceTransformerWrapper:
    from .models.sentence_transformer_wrapper import SentenceTransformerWrapper

    return SentenceTransformerWrapper(model=model_name, revision=revision, **kwargs)


def get_loader_name(
    loader: Callable[..., Encoder] | None,
) -> str | None:
    if loader is None:
        return None
    if hasattr(loader, "func"):  # partial class wrapper
        return loader.func.__name__
    return loader.__name__


class ModelMeta(BaseModel):
    """The model metadata object.

    Attributes:
        loader: the function that loads the model. If None it will just default to loading the model using the sentence transformer library.
        name: The name of the model, ideally the name on huggingface. It should be in the format "organization/model_name".
        n_parameters: The number of parameters in the model, e.g. 7_000_000 for a 7M parameter model. Can be None if the number of parameters is not known (e.g. for proprietary models) or
            if the loader returns a SentenceTransformer model from which it can be derived.
        memory_usage_mb: The memory usage of the model in MB. Can be None if the memory usage is not known (e.g. for proprietary models). To calculate it use the `calculate_memory_usage_mb` method.
        max_tokens: The maximum number of tokens the model can handle. Can be None if the maximum number of tokens is not known (e.g. for proprietary
            models).
        embed_dim: The dimension of the embeddings produced by the model. Currently all models are assumed to produce fixed-size embeddings.
        revision: The revision number of the model. If None, it is assumed that the metadata (including the loader) is valid for all revisions of the model.
        release_date: The date the model's revision was released.
        license: The license under which the model is released. Required if open_weights is True.
        open_weights: Whether the model is open source or proprietary.
        public_training_code: A link to the publicly available training code. If None, it is assumed that the training code is not publicly available.
        public_training_data: A link to the publicly available training data. If None, it is assumed that the training data is not publicly available.
        similarity_fn_name: The distance metric used by the model.
        framework: The framework the model is implemented in, can be a list of frameworks e.g. `["Sentence Transformers", "PyTorch"]`.
        reference: A URL to the model's page on huggingface or another source.
        languages: The languages the model is intended to be specified as a 3-letter language code followed by a script code e.g., "eng-Latn" for English
            in the Latin script.
        use_instructions: Whether the model uses instructions E.g. for prompt-based models. This also includes models that require a specific format for
            input, such as "query: {document}" or "passage: {document}".
        training_datasets: A dictionary of datasets that the model was trained on. Names should be names as they appear in `mteb` for example
            {"ArguAna": ["test"]} if the model is trained on the ArguAna test set. This field is used to determine if a model generalizes zero-shot to
            a benchmark as well as mark dataset contaminations.
        adapted_from: Name of the model from which this model is adapted. For quantizations, fine-tunes, long doc extensions, etc.
        superseded_by: Name of the model that supersedes this model, e.g., nvidia/NV-Embed-v2 supersedes v1.
        is_cross_encoder: Whether the model can act as a cross-encoder or not.
        modalities: A list of strings representing the modalities the model supports. Default is ["text"].
    """

    model_config = ConfigDict(extra="forbid")

    name: str | None
    revision: str | None
    release_date: STR_DATE | None
    languages: list[ISO_LANGUAGE_SCRIPT] | None
    loader: Callable[..., Encoder] | None = None
    n_parameters: int | None
    memory_usage_mb: float | None
    max_tokens: float | None
    embed_dim: int | None
    license: LICENSES | STR_URL | None
    open_weights: bool | None
    public_training_code: str | None
    public_training_data: str | bool | None
    framework: list[FRAMEWORKS]
    reference: STR_URL | None = None
    similarity_fn_name: DISTANCE_METRICS | None
    use_instructions: bool | None
    training_datasets: dict[str, list[str]] | None
    adapted_from: str | None = None
    superseded_by: str | None = None
    is_cross_encoder: bool | None = None
    modalities: list[MODALITIES] = ["text"]

    def to_dict(self):
        dict_repr = self.model_dump()
        loader = dict_repr.pop("loader", None)
        dict_repr["loader"] = get_loader_name(loader)
        return dict_repr

    @field_validator("languages")
    @classmethod
    def languages_are_valid(cls, languages: list[ISO_LANGUAGE_SCRIPT] | None) -> None:
        if languages is None:
            return None

        for code in languages:
            check_language_code(code)
        return languages

    @field_validator("name")
    @classmethod
    def check_name(cls, v: str | None) -> str | None:
        if v is None or v == "bm25s":
            return v
        if "/" not in v:
            raise ValueError(
                "Model name must be in the format 'organization/model_name'"
            )
        return v

    def load_model(self, **kwargs: Any) -> Encoder:
        if self.loader is None:
            logger.warning(
                f"Loader not specified for model {self.name}, loading using sentence transformers."
            )
            loader = partial(
                sentence_transformers_loader,
                model_name=self.name,
                revision=self.revision,
                **kwargs,
            )
        else:
            loader = self.loader

        model: Encoder = loader(**kwargs)  # type: ignore
        model.mteb_model_meta = self
        return model

    def model_name_as_path(self) -> str:
        if self.name is None:
            raise ValueError("Model name is not set")
        return self.name.replace("/", "__").replace(" ", "_")

    def is_zero_shot_on(self, tasks: Sequence[AbsTask] | Sequence[str]) -> bool | None:
        """Indicates whether the given model can be considered
        zero-shot or not on the given tasks.
        Returns None if no training data is specified on the model.
        """
        # If no tasks were specified, we're obviously zero-shot
        if not tasks:
            return True
        training_datasets = self.get_training_datasets()
        # If no tasks were specified, we're obviously zero-shot
        if training_datasets is None:
            return None
        model_datasets = {ds_name for ds_name, splits in training_datasets.items()}
        if isinstance(tasks[0], str):
            benchmark_datasets = set(tasks)
        else:
            tasks = cast(Sequence[AbsTask], tasks)
            benchmark_datasets = set()
            for task in tasks:
                benchmark_datasets.add(task.metadata.name)
        intersection = model_datasets & benchmark_datasets
        return len(intersection) == 0

    def get_training_datasets(self) -> dict[str, list[str]] | None:
        """Returns all training datasets of the model including similar tasks."""
        import mteb

        if self.training_datasets is None:
            return None

        training_datasets = self.training_datasets.copy()
        if self.adapted_from is not None:
            try:
                adapted_from_model = mteb.get_model_meta(
                    self.adapted_from, fetch_from_hf=False
                )
                adapted_training_datasets = adapted_from_model.get_training_datasets()
                if adapted_training_datasets is not None:
                    training_datasets |= adapted_training_datasets
            except ValueError as e:
                logger.warning(f"Could not get source model: {e} in MTEB")

        return_dataset = training_datasets.copy()
        visited = set()

        for dataset in training_datasets:
            similar_tasks = collect_similar_tasks(dataset, visited)
            return_dataset |= {task: [] for task in similar_tasks}

        return return_dataset

    def zero_shot_percentage(
        self, tasks: Sequence[AbsTask] | Sequence[str]
    ) -> int | None:
        """Indicates how out-of-domain the selected tasks are for the given model."""
        training_datasets = self.get_training_datasets()
        if (training_datasets is None) or (not tasks):
            return None
        model_datasets = {ds_name for ds_name, splits in training_datasets.items()}
        if isinstance(tasks[0], str):
            benchmark_datasets = set(tasks)
        else:
            tasks = cast(Sequence[AbsTask], tasks)
            benchmark_datasets = {task.metadata.name for task in tasks}
        overlap = model_datasets & benchmark_datasets
        perc_overlap = 100 * (len(overlap) / len(benchmark_datasets))
        return int(100 - perc_overlap)

    def calculate_memory_usage_mb(self) -> int | None:
        """Calculates the memory usage (in FP32) of the model in MB."""
        if "API" in self.framework:
            return None

        MB = 1024**2
        try:
            safetensors_metadata = get_safetensors_metadata(self.name)
            if len(safetensors_metadata.parameter_count) >= 0:
                dtype_size_map = {
                    "F64": 8,  # 64-bit float
                    "F32": 4,  # 32-bit float (FP32)
                    "F16": 2,  # 16-bit float (FP16)
                    "BF16": 2,  # BFloat16
                    "I64": 8,  # 64-bit integer
                    "I32": 4,  # 32-bit integer
                    "I16": 2,  # 16-bit integer
                    "I8": 1,  # 8-bit integer
                    "U8": 1,  # Unsigned 8-bit integer
                    "BOOL": 1,  # Boolean (assuming 1 byte per value)
                }
                total_memory_bytes = sum(
                    parameters * dtype_size_map.get(dtype, 4)
                    for dtype, parameters in safetensors_metadata.parameter_count.items()
                )
                return round(total_memory_bytes / MB)  # Convert to MB

        except (NotASafetensorsRepoError, SafetensorsParsingError, GatedRepoError):
            pass
        if self.n_parameters is None:
            return None
        # Model memory in bytes. For FP32 each parameter is 4 bytes.
        model_memory_bytes = self.n_parameters * 4

        # Convert to MB
        model_memory_mb = model_memory_bytes / MB
        return round(model_memory_mb)


def collect_similar_tasks(dataset: str, visited: set[str]) -> set[str]:
    """Recursively collect all similar tasks for a given dataset."""
    from .overview import SIMILAR_TASKS

    if dataset in visited:
        return set()

    visited.add(dataset)
    similar = set()

    # Check if dataset is a key in SIMILAR_TASKS
    if dataset in SIMILAR_TASKS:
        for similar_task in SIMILAR_TASKS[dataset]:
            similar.add(similar_task)
            similar.update(collect_similar_tasks(similar_task, visited))

    # Check if dataset appears as a value in SIMILAR_TASKS
    for parent, children in SIMILAR_TASKS.items():
        if dataset in children:
            similar.add(parent)
            similar.update(collect_similar_tasks(parent, visited))

    return similar



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskSTS import AbsTaskSTS


class JSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="JSTS",
        dataset={
            "path": "shunk031/JGLUE",
            "revision": "50e79c314a7603ebc92236b66a0973d51a00ed8c",
            "name": "JSTS",
            "trust_remote_code": True,
        },
        description="Japanese Semantic Textual Similarity Benchmark dataset construct from YJ Image Captions Dataset "
        + "(Miyazaki and Shimizu, 2016) and annotated by crowdsource annotators.",
        reference="https://aclanthology.org/2022.lrec-1.317.pdf#page=2.00",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["validation"],
        eval_langs=["jpn-Jpan"],
        main_score="cosine_spearman",
        date=("2016-01-01", "2022-12-31"),
        domains=["Web", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{kurihara-etal-2022-jglue,
    title = "{JGLUE}: {J}apanese General Language Understanding Evaluation",
    author = "Kurihara, Kentaro  and
      Kawahara, Daisuke  and
      Shibata, Tomohide",
    editor = "Calzolari, Nicoletta  and
      B{\'e}chet, Fr{\'e}d{\'e}ric  and
      Blache, Philippe  and
      Choukri, Khalid  and
      Cieri, Christopher  and
      Declerck, Thierry  and
      Goggi, Sara  and
      Isahara, Hitoshi  and
      Maegaard, Bente  and
      Mariani, Joseph  and
      Mazo, H{\'e}l{\`e}ne  and
      Odijk, Jan  and
      Piperidis, Stelios",
    booktitle = "Proceedings of the Thirteenth Language Resources and Evaluation Conference",
    month = jun,
    year = "2022",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2022.lrec-1.317",
    pages = "2957--2966",
    abstract = "To develop high-performance natural language understanding (NLU) models, it is necessary to have a benchmark to evaluate and analyze NLU ability from various perspectives. While the English NLU benchmark, GLUE, has been the forerunner, benchmarks are now being released for languages other than English, such as CLUE for Chinese and FLUE for French; but there is no such benchmark for Japanese. We build a Japanese NLU benchmark, JGLUE, from scratch without translation to measure the general NLU ability in Japanese. We hope that JGLUE will facilitate NLU research in Japanese.",
}""",
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 5
        return metadata_dict

    def dataset_transform(self) -> None:
        self.dataset = self.dataset.rename_column("label", "score")



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import numpy as np

from mteb.abstasks.AbsTask import AbsTask
from mteb.abstasks.aggregate_task_metadata import AggregateTaskMetadata

if TYPE_CHECKING:
    from datasets import Dataset, DatasetDict

    from mteb.abstasks.TaskMetadata import DescriptiveStatistics, HFSubset
    from mteb.encoder_interface import Encoder
    from mteb.load_results.task_results import TaskResult

    from .AbsTask import ScoresDict

logger = logging.getLogger(__name__)


class AbsTaskAggregate(AbsTask):
    metadata: AggregateTaskMetadata
    superseded_by: None | str = None
    hf_subset = "default"  # since there is no subset we use the "default" naming scheme
    _eval_splits: list[str] | None = None

    def __init__(self, **kwargs: Any):
        self.tasks = self.metadata.tasks
        self.taskname_to_task = {task.metadata.name: task for task in self.tasks}

    def task_results_to_scores(
        self, task_results: list[TaskResult]
    ) -> dict[str, dict[HFSubset, ScoresDict]]:
        """The function that aggregated scores. Can be redefined to allow for custom aggregations."""
        scores = {}
        subsets = (
            self.metadata.eval_langs.keys()
            if isinstance(self.metadata.eval_langs, dict)
            else None
        )
        eval_langs = (
            self.metadata.eval_langs.values()
            if isinstance(self.metadata.eval_langs, dict)
            else [self.metadata.eval_langs]
        )
        for split in self.metadata.eval_splits:
            main_scores = []
            for task_res in task_results:
                for langs in eval_langs:
                    main_scores.append(
                        task_res.get_score_fast(
                            languages=[lang.split("-")[0] for lang in langs],
                            splits=self.metadata.eval_splits,
                            subsets=subsets,
                        )
                    )
            main_score = np.mean(main_scores)
            scores[split] = {
                "default": {
                    self.metadata.main_score: main_score,
                    "main_score": main_score,
                }
            }
        return scores

    def combine_task_results(self, task_results: list[TaskResult]) -> TaskResult:
        """Combined the task results for using `task_results_to_scores`. Do not redefine this function if you want to implement a custom aggregation.
        Instead redefin `task_results_to_scores`.
        """
        from mteb.load_results.task_results import (
            TaskResult,  # to prevent circular imports, # TODO: can potentially likely be out of function in in v2.0.0
        )

        eval_times = [tr.evaluation_time for tr in task_results if tr.evaluation_time]
        if len(eval_times) != len(task_results):
            logger.info(
                f"Loaded results does not include runtime. Therefore evaluation of {self.metadata.name} "
                + "can't be computed. Setting it to None."
            )
            eval_time = np.nan
        else:
            eval_time = sum(eval_times)

        kg_co2_emissions_ = [
            tr.kg_co2_emissions for tr in task_results if tr.kg_co2_emissions
        ]
        if len(kg_co2_emissions_) != len(task_results):
            logger.info(
                f"Loaded results does not include co2-eq emissions. Therefore evaluation of {self.metadata.name} "
                + "can't be computed. Setting it to None."
            )
            kg_co2_emissions = np.nan
        else:
            kg_co2_emissions = sum(kg_co2_emissions_)

        task_res = TaskResult.from_task_results(
            self,
            scores=self.task_results_to_scores(task_results),
            evaluation_time=eval_time,
            kg_co2_emissions=kg_co2_emissions,
        )
        mteb_versions = {tr.mteb_version for tr in task_results}
        if len(mteb_versions) != 1:
            logger.warning(
                f"All tasks of {self.metadata.name} is not run using the same version."
            )
            task_res.mteb_version = None
        task_res.mteb_version = task_results[0].mteb_version
        return task_res

    def check_if_dataset_is_superseded(self):
        """Check if the dataset is superseded by a newer version"""
        if self.superseded_by:
            logger.warning(
                f"Dataset '{self.metadata.name}' is superseded by '{self.superseded_by}', you might consider using the newer version of the dataset."
            )

    def filter_eval_splits(self, eval_splits: list[str] | None) -> AbsTaskAggregate:
        """Filter the evaluation splits of the task."""
        self._eval_splits = eval_splits
        return self

    def evaluate(
        self,
        model: Encoder,
        split: str = "test",
        subsets_to_run: list[HFSubset] | None = None,
        *,
        encode_kwargs: dict[str, Any] = {},
        **kwargs: Any,
    ) -> dict[HFSubset, ScoresDict]:
        # TODO: If we refactor the runner to at least have a subfunction mteb.run_task(model, task) we could use that here
        raise NotImplementedError(
            "Aggregate tasks can't be evaluated directly. Instead run it using the MTEB class."
        )

    def _evaluate_subset(
        self,
        model: Encoder,
        data_split: DatasetDict | Dataset,
        encode_kwargs: dict[str, Any],
        **kwargs: Any,
    ) -> ScoresDict:
        raise NotImplementedError(
            "Aggregate tasks does not implement a _evaluate_subset. Instead use the individual tasks."
        )

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> DescriptiveStatistics:
        raise NotImplementedError(
            "Aggregate tasks does not implement a _calculate_metrics_from_split. Instead use the individual tasks."
        )

    @property
    def is_aggregate(self):  # Overrides the is_aggregate method on AbsTask
        return True

    @property
    def eval_splits(self) -> list[str]:
        if self._eval_splits:
            return self._eval_splits
        return self.metadata.eval_splits



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskZeroShotClassification import (
    AbsTaskZeroShotClassification,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class RESISC45ZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="RESISC45ZeroShot",
        description="Remote Sensing Image Scene Classification by Northwestern Polytechnical University (NWPU).",
        reference="https://ieeexplore.ieee.org/abstract/document/7891544",
        dataset={
            "path": "timm/resisc45",
            "revision": "fe12fc5f1b7606543b0355eda392f1ddc54625c6",
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2017-01-01",
            "2017-03-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image", "text"],
        sample_creation="created",
        bibtex_citation="""@ARTICLE{7891544,
        author={Cheng, Gong and Han, Junwei and Lu, Xiaoqiang},
        journal={Proceedings of the IEEE},
        title={Remote Sensing Image Scene Classification: Benchmark and State of the Art},
        year={2017},
        volume={105},
        number={10},
        pages={1865-1883},
        keywords={Remote sensing;Benchmark testing;Spatial resolution;Social network services;Satellites;Image analysis;Machine learning;Unsupervised learning;Classification;Benchmark data set;deep learning;handcrafted features;remote sensing image;scene classification;unsupervised feature learning},
        doi={10.1109/JPROC.2017.2675998}}""",
        descriptive_stats={
            "n_samples": {"test": 6300},
            "avg_character_length": {"test": 256},
        },
    )

    def get_candidate_labels(self) -> list[str]:
        return [
            f"satellite imagery of {name}."
            for name in self.dataset["test"].features[self.label_column_name].names
        ]



from __future__ import annotations

from mteb.abstasks.AbsTaskBitextMining import AbsTaskBitextMining
from mteb.abstasks.TaskMetadata import TaskMetadata


class NorwegianCourtsBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="NorwegianCourtsBitextMining",
        dataset={
            "path": "kardosdrur/norwegian-courts",
            "revision": "d79af07e969a6678fcbbe819956840425816468f",
        },
        description="Nynorsk and Bokmål parallel corpus from Norwegian courts. Norwegian courts have two standardised written languages. Bokmål is a variant closer to Danish, while Nynorsk was created to resemble regional dialects of Norwegian.",
        reference="https://opus.nlpl.eu/index.php",
        type="BitextMining",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nob-Latn", "nno-Latn"],
        main_score="f1",
        date=("2020-01-01", "2020-12-31"),
        domains=["Legal", "Written"],
        task_subtypes=[],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
@inproceedings{opus4,
  title={OPUS-MT — Building open translation services for the World},
  author={Tiedemann, J{\"o}rg and Thottingal, Santhosh},
  booktitle={Proceedings of the 22nd Annual Conference of the European Association for Machine Translation (EAMT)},
  year={2020}
}
""",
        prompt="Retrieve parallel sentences in Norwegian Bokmål and Nynorsk",
    )

    def dataset_transform(self):
        # Convert to standard format
        self.dataset = self.dataset.rename_column("nb", "sentence1")
        self.dataset = self.dataset.rename_column("nn", "sentence2")



from __future__ import annotations

import shutil

import numpy as np
import pytest

from mteb.encoder_interface import Encoder
from mteb.models.cache_wrapper import CachedEmbeddingWrapper


class DummyModel(Encoder):
    def __init__(self, embedding_dim=768):
        self.embedding_dim = embedding_dim
        self.call_count = 0

    def encode(self, texts, **kwargs):
        self.call_count += 1
        return np.random.rand(len(texts), self.embedding_dim).astype(np.float32)

    def random_other_function_returns_false(self):
        return False


class TestCachedEmbeddingWrapper:
    @pytest.fixture(scope="function")
    def cache_dir(self, tmp_path):
        cache_path = tmp_path / "test_cache"
        yield cache_path
        # Cleanup after test
        if cache_path.exists():
            shutil.rmtree(cache_path)

    def test_caching_functionality(self, cache_dir):
        # Create a dummy model
        dummy_model = DummyModel()

        # Create the wrapper
        wrapped_model = CachedEmbeddingWrapper(dummy_model, cache_dir)

        # Simulate data
        queries = [
            "What is the effect of vitamin C on common cold?",
            "How does exercise affect cardiovascular health?",
        ]
        corpus = [
            "Vitamin C supplementation does not significantly reduce the incidence of common cold.",
            "Regular exercise improves cardiovascular health by strengthening the heart and reducing blood pressure.",
            "The impact of vitamin C on common cold duration is minimal according to recent studies.",
        ]

        # First call - should use the model to compute embeddings
        query_embeddings1 = wrapped_model.encode(queries, task_name="DummyTaskQuery")
        corpus_embeddings1 = wrapped_model.encode(corpus, task_name="DummyTaskCorpus")

        assert dummy_model.call_count == 2  # One call for queries, one for corpus

        # Second call - should use cached embeddings
        query_embeddings2 = wrapped_model.encode(queries, task_name="DummyTaskQuery")
        corpus_embeddings2 = wrapped_model.encode(corpus, task_name="DummyTaskCorpus")

        assert dummy_model.call_count == 2  # No additional calls to the model

        # Verify that the embeddings are the same
        np.testing.assert_allclose(query_embeddings1, query_embeddings2)
        np.testing.assert_allclose(corpus_embeddings1, corpus_embeddings2)

        # Verify that cache files were created
        assert (cache_dir / "DummyTaskQuery" / "vectors.npy").exists()
        assert (cache_dir / "DummyTaskQuery" / "index.json").exists()
        assert (cache_dir / "DummyTaskCorpus" / "vectors.npy").exists()
        assert (cache_dir / "DummyTaskCorpus" / "index.json").exists()

        # Test with a new query - should use cache for existing queries and compute for new one
        new_queries = ["What is the role of insulin in diabetes?"]
        query_embeddings3 = wrapped_model.encode(
            new_queries, task_name="DummyTaskQuery"
        )

        assert dummy_model.call_count == 3  # One additional call for the new query
        assert query_embeddings3.shape == (1, dummy_model.embedding_dim)

        # try with a cached query only
        _ = wrapped_model.encode(queries, task_name="DummyTaskQuery")
        assert dummy_model.call_count == 3

        wrapped_model.close()  # delete to allow cleanup on Windows

    def test_other_functions_still_work(self, cache_dir):
        # Create a dummy model
        dummy_model = DummyModel()

        # Create the wrapper
        wrapped_model = CachedEmbeddingWrapper(dummy_model, cache_dir)

        # Call a function that is not wrapped
        result = wrapped_model.random_other_function_returns_false()

        assert result is False
        assert wrapped_model.call_count == 0

        wrapped_model.close()  # delete to allow cleanup on Windows



from __future__ import annotations

import logging
import math
import os
from typing import Any

import torch
from sklearn import metrics
from torch.utils.data import DataLoader

from mteb.encoder_interface import Encoder
from mteb.requires_package import requires_image_dependencies

from ..Evaluator import Evaluator

logger = logging.getLogger(__name__)


def get_default_transform():
    requires_image_dependencies()
    from torchvision import transforms

    return transforms.Compose([transforms.PILToTensor()])


class ImageDataset(torch.utils.data.Dataset):
    def __init__(self, hf_dataset, image_column_name: str = "image", transform=None):
        self.dataset = hf_dataset
        self.transform = transform
        self.image_column_name = image_column_name

    def __len__(self):
        return len(self.dataset)

    def __getitem__(self, idx):
        image = self.dataset[idx][self.image_column_name]
        if image.mode != "RGB":
            image = image.convert("RGB")
        image = self.transform(image)
        return image


def custom_collate_fn(batch):
    return batch


class ZeroShotClassificationEvaluator(Evaluator):
    def __init__(
        self,
        dataset,
        image_column_name: str,
        labels: list[int],
        candidate_labels: list[str],
        task_name: str | None = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.dataset = ImageDataset(
            dataset,
            image_column_name=image_column_name,
            transform=get_default_transform(),
        )
        self.image_column_name = image_column_name
        self.labels = labels
        self.candidate_labels = candidate_labels
        self.task_name = task_name

    def __call__(self, model: Encoder, *, encode_kwargs: dict[str, Any] = {}):
        if "batch_size" not in encode_kwargs:
            encode_kwargs["batch_size"] = 32

        dataloader = DataLoader(
            self.dataset,
            batch_size=encode_kwargs["batch_size"],
            shuffle=False,
            collate_fn=custom_collate_fn,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )

        text_embeddings = model.get_text_embeddings(
            self.candidate_labels, batch_size=encode_kwargs["batch_size"]
        )

        image_embeddings = model.get_image_embeddings(
            dataloader, batch_size=encode_kwargs["batch_size"]
        )

        probs = model.calculate_probs(text_embeddings, image_embeddings)
        predictions = probs.argmax(dim=1)

        logger.info("Evaluating...")

        accuracy = metrics.accuracy_score(self.labels, predictions.tolist())

        return {"accuracy": accuracy}



from __future__ import annotations

import datasets

from mteb.abstasks.AbsTaskRetrieval import AbsTaskRetrieval
from mteb.abstasks.TaskMetadata import TaskMetadata


def split_by_first_newline(s):
    # Split the string by the first newline
    parts = s.split("\n", 1)
    # Return parts or (s, '') if no newline
    return parts if len(parts) > 1 else (s, "")


common_args = {
    "reference": "https://arxiv.org/pdf/2406.14497",
    "type": "Reranking",
    "category": "s2s",
    "modalities": ["text"],
    "eval_splits": ["train"],
    "eval_langs": ["python-Code"],
    "main_score": "ndcg_at_10",
    "date": ("2024-06-02", "2024-06-02"),  # best guess
    "domains": ["Programming"],
    "task_subtypes": ["Code retrieval"],
    "license": "cc-by-sa-4.0",
    "annotations_creators": "derived",
    "dialect": [],
    "sample_creation": "found",
    "bibtex_citation": """
    @misc{wang2024coderagbenchretrievalaugmentcode,
        title={CodeRAG-Bench: Can Retrieval Augment Code Generation?},
        author={Zora Zhiruo Wang and Akari Asai and Xinyan Velocity Yu and Frank F. Xu and Yiqing Xie and Graham Neubig and Daniel Fried},
        year={2024},
        eprint={2406.14497},
        archivePrefix={arXiv},
        primaryClass={cs.SE},
        url={https://arxiv.org/abs/2406.14497},
    }
    """,
}


class CodeRAGProgrammingSolutionsRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CodeRAGProgrammingSolutions",
        description="Evaluation of programming solution retrieval using CodeRAG-Bench. Tests the ability to retrieve relevant programming solutions given code-related queries.",
        dataset={
            "path": "code-rag-bench/programming-solutions",
            "revision": "1064f7bba54d5400d4836f5831fe4c2332a566a6",
        },
        **common_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(**self.metadata.dataset)  # type: ignore
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """And transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}

        split = self.metadata.eval_splits[0]
        ds: datasets.Dataset = self.dataset[split]  # type: ignore
        ds = ds.shuffle(seed=42)

        self.queries[split] = {}
        self.relevant_docs[split] = {}
        self.corpus[split] = {}

        texts = ds["text"]
        meta = ds["meta"]
        for text, mt in zip(texts, meta):
            # in code-rag-bench,
            # text = query + "\n" + doc(code)
            query, doc = split_by_first_newline(text)

            id = mt["task_id"]

            query_id = id
            doc_id = f"doc_{id}"
            self.queries[split][query_id] = query
            self.corpus[split][doc_id] = {"title": "", "text": doc}

            self.relevant_docs[split][query_id] = {
                doc_id: 1
            }  # only one correct matches


class CodeRAGOnlineTutorialsRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CodeRAGOnlineTutorials",
        description="Evaluation of online programming tutorial retrieval using CodeRAG-Bench. Tests the ability to retrieve relevant tutorials from online platforms given code-related queries.",
        dataset={
            "path": "code-rag-bench/online-tutorials",
            "revision": "095bb77130082e4690d6c3a031997b03487bf6e2",
        },
        **common_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(**self.metadata.dataset)  # type: ignore
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """And transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}

        split = self.metadata.eval_splits[0]
        ds: datasets.Dataset = self.dataset[split]  # type: ignore
        ds = ds.shuffle(seed=42)

        self.queries[split] = {}
        self.relevant_docs[split] = {}
        self.corpus[split] = {}

        titles = ds["title"]
        texts = ds["text"]
        parsed = ds["parsed"]
        id = 0
        for title, text, mt in zip(titles, texts, parsed):
            # in code-rag-bench,
            # query=doc(code)
            # text=query+doc(code)
            query, doc = title, text

            query_id = str(id)
            doc_id = f"doc_{id}"
            self.queries[split][query_id] = query
            self.corpus[split][doc_id] = {"title": "", "text": doc}

            self.relevant_docs[split][query_id] = {
                doc_id: 1
            }  # only one correct matches

            id += 1


class CodeRAGLibraryDocumentationSolutionsRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CodeRAGLibraryDocumentationSolutions",
        description="Evaluation of code library documentation retrieval using CodeRAG-Bench. Tests the ability to retrieve relevant Python library documentation sections given code-related queries.",
        dataset={
            "path": "code-rag-bench/library-documentation",
            "revision": "b530d3b5a25087d2074e731b76232db85b9e9107",
        },
        **common_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(**self.metadata.dataset)  # type: ignore
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """And transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}

        split = self.metadata.eval_splits[0]
        ds: datasets.Dataset = self.dataset[split]  # type: ignore
        ds = ds.shuffle(seed=42)

        self.queries[split] = {}
        self.relevant_docs[split] = {}
        self.corpus[split] = {}

        texts = ds["doc_content"]

        id = 0
        for text in texts:
            # text format "document title \n document content"
            query, doc = split_by_first_newline(text)

            # some library documents doesn't have query-doc pair
            if not doc:
                continue
            query_id = str(id)
            doc_id = f"doc_{id}"
            self.queries[split][query_id] = query
            self.corpus[split][doc_id] = {"title": "", "text": doc}
            # only one correct match
            self.relevant_docs[split][query_id] = {doc_id: 1}
            id += 1


class CodeRAGStackoverflowPostsRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CodeRAGStackoverflowPosts",
        description="Evaluation of StackOverflow post retrieval using CodeRAG-Bench. Tests the ability to retrieve relevant StackOverflow posts given code-related queries.",
        dataset={
            "path": "code-rag-bench/stackoverflow-posts",
            "revision": "04e05d86cb0ac467b29a5d87f4c56eac99dfc0a4",
        },
        **common_args,  # type: ignore
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.dataset = datasets.load_dataset(**self.metadata.dataset)  # type: ignore
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """And transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}

        split = self.metadata.eval_splits[0]
        ds: datasets.Dataset = self.dataset[split]  # type: ignore
        ds = ds.shuffle(seed=42)

        self.queries[split] = {}
        self.relevant_docs[split] = {}
        self.corpus[split] = {}

        texts = ds["text"]
        id = 0
        for text in texts:
            # in code-rag-bench,
            # text = query + "\n" + doc
            query, doc = split_by_first_newline(text)

            query_id = str(id)
            doc_id = f"doc_{id}"
            self.queries[split][query_id] = query
            self.corpus[split][doc_id] = {"title": "", "text": doc}

            self.relevant_docs[split][query_id] = {
                doc_id: 1
            }  # only one correct matches
            id += 1



from __future__ import annotations

import logging
import math
import os
from typing import Any

import numpy as np
import torch
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import (
    accuracy_score,
    average_precision_score,
    f1_score,
)
from sklearn.neighbors import KNeighborsClassifier
from torch import Tensor
from torch.utils.data import DataLoader

from mteb.encoder_interface import Encoder
from mteb.requires_package import requires_image_dependencies

from ..Evaluator import Evaluator

logger = logging.getLogger(__name__)


def dot_distance(a: np.ndarray, b: np.ndarray) -> float:
    return -np.dot(a, b)


def get_default_transform():
    requires_image_dependencies()
    from torchvision import transforms

    return transforms.Compose([transforms.PILToTensor()])


class ImageDataset(torch.utils.data.Dataset):
    def __init__(self, hf_dataset, image_column_name: str = "image", transform=None):
        self.dataset = hf_dataset
        self.transform = transform
        self.image_column_name = image_column_name

    def __len__(self):
        return len(self.dataset)

    def __getitem__(self, idx):
        image = self.dataset[idx][self.image_column_name]
        if image.mode != "RGB":
            image = image.convert("RGB")
        image = self.transform(image)
        return image


def custom_collate_fn(batch):
    return batch


class ImagekNNClassificationEvaluator(Evaluator):
    def __init__(
        self,
        dataset_train,
        dataset_test,
        image_column_name,
        label_column_name,
        task_name: str | None = None,
        k: int = 1,
        encode_kwargs: dict[str, Any] = {},
        limit: int | None = None,
        **kwargs,
    ):
        super().__init__(**kwargs)

        if limit is not None:
            dataset_train = dataset_train.select(list(range(limit)))

        default_transform = get_default_transform()
        self.dataset_train = ImageDataset(
            dataset_train,
            image_column_name=image_column_name,
            transform=default_transform,
        )
        self.y_train = dataset_train[label_column_name]

        self.dataset_test = ImageDataset(
            dataset_test,
            image_column_name=image_column_name,
            transform=default_transform,
        )
        self.y_test = dataset_test[label_column_name]
        self.task_name = task_name
        self.encode_kwargs = encode_kwargs

        if "batch_size" not in self.encode_kwargs:
            self.encode_kwargs["batch_size"] = 32

        self.k = k

    def __call__(self, model, test_cache=None):
        scores = {}
        max_accuracy = 0
        max_f1 = 0
        max_ap = 0
        dataloader_train = DataLoader(
            self.dataset_train,
            batch_size=self.encode_kwargs["batch_size"],
            shuffle=False,
            collate_fn=custom_collate_fn,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )
        X_train = model.get_image_embeddings(
            dataloader_train, batch_size=self.encode_kwargs["batch_size"]
        )
        dataloader = DataLoader(
            self.dataset_test,
            batch_size=self.encode_kwargs["batch_size"],
            shuffle=False,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )
        if test_cache is None:
            X_test = model.get_image_embeddings(
                dataloader, batch_size=self.encode_kwargs["batch_size"]
            )
            test_cache = X_test
        else:
            X_test = test_cache
        for metric in ["cosine", "euclidean"]:  # TODO: "dot"
            knn = KNeighborsClassifier(n_neighbors=self.k, n_jobs=-1, metric=metric)
            knn.fit(X_train, self.y_train)
            y_pred = knn.predict(X_test)
            accuracy = accuracy_score(self.y_test, y_pred)
            f1 = f1_score(self.y_test, y_pred, average="macro")
            scores["accuracy_" + metric] = accuracy
            scores["f1_" + metric] = f1
            max_accuracy = max(max_accuracy, accuracy)
            max_f1 = max(max_f1, f1)  # type: ignore
            # if binary classification
            if len(np.unique(self.y_train)) == 2:
                ap = average_precision_score(self.y_test, y_pred)
                scores["ap_" + metric] = ap
                max_ap = max(max_ap, ap)
        scores["accuracy"] = max_accuracy
        scores["f1"] = max_f1
        if len(np.unique(self.y_train)) == 2:
            scores["ap"] = max_ap
        return scores, test_cache


class ImagekNNClassificationEvaluatorPytorch(Evaluator):
    def __init__(
        self,
        dataset_train,
        dataset_test,
        image_column_name,
        label_column_name,
        task_name: str,
        k: int = 1,
        encode_kwargs: dict[str, Any] = {},
        limit: int | None = None,
        **kwargs: Any,
    ):
        super().__init__(**kwargs)
        if limit is not None:
            dataset_train = dataset_train.select(list(range(limit)))

        default_transform = get_default_transform()
        self.dataset_train = ImageDataset(
            dataset_train,
            image_column_name=image_column_name,
            transform=default_transform,
        )
        self.y_train = dataset_train[label_column_name]

        self.dataset_test = ImageDataset(
            dataset_test,
            image_column_name=image_column_name,
            transform=default_transform,
        )
        self.y_test = dataset_test[label_column_name]
        self.task_name = task_name
        self.encode_kwargs = encode_kwargs

        if "batch_size" not in self.encode_kwargs:
            self.encode_kwargs["batch_size"] = 32

        self.k = k

    def __call__(self, model: Encoder, test_cache=None):
        scores = {}
        max_accuracy = 0
        max_f1 = 0
        max_ap = 0

        dataloader_train = DataLoader(
            self.dataset_train,
            batch_size=self.encode_kwargs["batch_size"],
            shuffle=False,
            collate_fn=custom_collate_fn,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )
        X_train = model.get_image_embeddings(
            dataloader_train, batch_size=self.encode_kwargs["batch_size"]
        )

        dataloader = DataLoader(
            self.dataset_test,
            batch_size=self.encode_kwargs["batch_size"],
            shuffle=False,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )
        if test_cache is None:
            X_test = model.get_image_embeddings(
                dataloader, batch_size=self.encode_kwargs["batch_size"]
            )
            test_cache = X_test
        else:
            X_test = test_cache
        for metric in ["cosine", "euclidean", "dot"]:
            if metric == "cosine":
                distances = 1 - self._cos_sim(X_test, X_train)
            elif metric == "euclidean":
                distances = self._euclidean_dist(X_test, X_train)
            elif metric == "dot":
                distances = -self._dot_score(X_test, X_train)
            neigh_indices = torch.topk(
                distances, k=self.k, dim=1, largest=False
            ).indices
            y_train = torch.tensor(self.y_train)
            y_pred = torch.mode(
                y_train[neigh_indices], dim=1
            ).values  # TODO: case where there is no majority
            y_pred = y_pred.tolist()
            accuracy = accuracy_score(self.y_test, y_pred)
            f1 = f1_score(self.y_test, y_pred, average="macro")
            scores["accuracy_" + metric] = accuracy
            scores["f1_" + metric] = f1
            max_accuracy = max(max_accuracy, accuracy)
            max_f1 = max(max_f1, f1)  # type: ignore
            # if binary classification
            if len(np.unique(self.y_train)) == 2:
                ap = average_precision_score(self.y_test, y_pred)
                scores["ap_" + metric] = ap
                max_ap = max(max_ap, ap)
        scores["accuracy"] = max_accuracy
        scores["f1"] = max_f1
        if len(np.unique(self.y_train)) == 2:
            scores["ap"] = max_ap
        return scores, test_cache

    @staticmethod
    def _cos_sim(a: Tensor, b: Tensor):
        """Computes the cosine similarity cos_sim(a[i], b[j]) for all i and j.

        Return:
            Matrix with res[i][j]  = cos_sim(a[i], b[j])
        """
        if not isinstance(a, torch.Tensor):
            a = torch.tensor(a)

        if not isinstance(b, torch.Tensor):
            b = torch.tensor(b)

        if len(a.shape) == 1:
            a = a.unsqueeze(0)

        if len(b.shape) == 1:
            b = b.unsqueeze(0)

        a_norm = torch.nn.functional.normalize(a, p=2, dim=1)
        b_norm = torch.nn.functional.normalize(b, p=2, dim=1)
        return torch.mm(a_norm, b_norm.transpose(0, 1))

    @staticmethod
    def _euclidean_dist(a: Tensor, b: Tensor):
        """Computes the euclidean distance euclidean_dist(a[i], b[j]) for all i and j.

        Returns:
            Matrix with res[i][j]  = euclidean_dist(a[i], b[j])
        """
        if not isinstance(a, torch.Tensor):
            a = torch.tensor(a)

        if not isinstance(b, torch.Tensor):
            b = torch.tensor(b)

        if len(a.shape) == 1:
            a = a.unsqueeze(0)

        if len(b.shape) == 1:
            b = b.unsqueeze(0)

        return torch.cdist(a, b, p=2)

    @staticmethod
    def _dot_score(a: Tensor, b: Tensor):
        """Computes the dot-product dot_prod(a[i], b[j]) for all i and j.

        Returns:
            Matrix with res[i][j]  = dot_prod(a[i], b[j])
        """
        if not isinstance(a, torch.Tensor):
            a = torch.tensor(a)

        if not isinstance(b, torch.Tensor):
            b = torch.tensor(b)

        if len(a.shape) == 1:
            a = a.unsqueeze(0)

        if len(b.shape) == 1:
            b = b.unsqueeze(0)

        return torch.mm(a, b.transpose(0, 1))


class ImagelogRegClassificationEvaluator(Evaluator):
    def __init__(
        self,
        dataset_train,
        dataset_test,
        image_column_name,
        label_column_name,
        task_name: str,
        max_iter: int = 100,
        encode_kwargs: dict[str, Any] = {},
        limit: int | None = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.encode_kwargs = encode_kwargs

        if "batch_size" not in self.encode_kwargs:
            self.encode_kwargs["batch_size"] = 32

        if limit is not None:
            dataset_train = dataset_train.select(list(range(limit)))

        default_transform = get_default_transform()
        self.dataset_train = ImageDataset(
            dataset_train,
            image_column_name=image_column_name,
            transform=default_transform,
        )
        self.y_train = dataset_train[label_column_name]
        self.dataset_test = ImageDataset(
            dataset_test,
            image_column_name=image_column_name,
            transform=default_transform,
        )
        self.y_test = dataset_test[label_column_name]

        self.max_iter = max_iter
        self.task_name = task_name

    def __call__(self, model, test_cache=None):
        scores = {}
        clf = LogisticRegression(
            random_state=self.seed,
            n_jobs=-1,
            max_iter=self.max_iter,
            verbose=1 if logger.isEnabledFor(logging.DEBUG) else 0,
        )
        dataloader_train = DataLoader(
            self.dataset_train,
            batch_size=self.encode_kwargs["batch_size"],
            shuffle=False,
            collate_fn=custom_collate_fn,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )
        X_train = model.get_image_embeddings(
            dataloader_train, batch_size=self.encode_kwargs["batch_size"]
        )
        dataloader = DataLoader(
            self.dataset_test,
            batch_size=self.encode_kwargs["batch_size"],
            shuffle=False,
            collate_fn=custom_collate_fn,
            num_workers=min(math.floor(os.cpu_count() / 2), 16),
        )
        if test_cache is None:
            X_test = model.get_image_embeddings(
                dataloader, batch_size=self.encode_kwargs["batch_size"]
            )
            test_cache = X_test
        else:
            X_test = test_cache
        logger.info("Fitting logistic regression classifier...")
        if X_train.dtype == torch.bfloat16:
            X_train = X_train.to(torch.float32)
        if X_test.dtype == torch.bfloat16:
            X_test = X_test.to(torch.float32)
        clf.fit(X_train, self.y_train)
        logger.info("Evaluating...")
        y_pred = clf.predict(X_test)
        scores["accuracy"] = accuracy_score(self.y_test, y_pred)
        scores["f1"] = f1_score(self.y_test, y_pred, average="macro")
        scores["f1_weighted"] = f1_score(self.y_test, y_pred, average="weighted")

        # if binary classification
        if len(np.unique(self.y_train)) == 2:
            scores["ap"] = average_precision_score(self.y_test, y_pred, average="macro")
            scores["ap_weighted"] = average_precision_score(
                self.y_test, y_pred, average="weighted"
            )

        return scores, test_cache



from __future__ import annotations

from collections import defaultdict

from datasets import DatasetDict, load_dataset

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


def load_retrieval_data(dataset_path, dataset_revision, qrel_revision, eval_splits):
    eval_split = eval_splits[0]
    dataset = load_dataset(dataset_path, revision=dataset_revision)
    qrels = load_dataset(dataset_path + "-qrels", revision=qrel_revision)[eval_split]

    corpus = {e["id"]: {"text": e["text"]} for e in dataset["corpus"]}
    queries = {e["id"]: e["text"] for e in dataset["queries"]}
    relevant_docs = defaultdict(dict)
    for e in qrels:
        relevant_docs[e["qid"]][e["pid"]] = e["score"]

    corpus = DatasetDict({eval_split: corpus})
    queries = DatasetDict({eval_split: queries})
    relevant_docs = DatasetDict({eval_split: relevant_docs})
    return corpus, queries, relevant_docs


class T2Retrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="T2Retrieval",
        description="T2Ranking: A large-scale Chinese Benchmark for Passage Ranking",
        reference="https://arxiv.org/abs/2304.03679",
        dataset={
            "path": "C-MTEB/T2Retrieval",
            "revision": "8731a845f1bf500a4f111cf1070785c793d10e64",
            "qrel_revision": "1c83b8d1544e529875e3f6930f3a1fcf749a8e97",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{xie2023t2ranking,
      title={T2Ranking: A large-scale Chinese Benchmark for Passage Ranking},
      author={Xiaohui Xie and Qian Dong and Bingning Wang and Feiyang Lv and Ting Yao and Weinan Gan and Zhijing Wu and Xiangsheng Li and Haitao Li and Yiqun Liu and Jin Ma},
      year={2023},
      eprint={2304.03679},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        prompt={
            "query": "Given a Chinese search query, retrieve web passages that answer the question"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class MMarcoRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MMarcoRetrieval",
        description="MMarcoRetrieval",
        reference="https://arxiv.org/abs/2309.07597",
        dataset={
            "path": "C-MTEB/MMarcoRetrieval",
            "revision": "539bbde593d947e2a124ba72651aafc09eb33fc2",
            "qrel_revision": "bae08bb7bddbedb96c7e7db52018a55167b67f89",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{xiao2024cpack,
      title={C-Pack: Packaged Resources To Advance General Chinese Embedding},
      author={Shitao Xiao and Zheng Liu and Peitian Zhang and Niklas Muennighoff and Defu Lian and Jian-Yun Nie},
      year={2024},
      eprint={2309.07597},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        prompt={
            "query": "Given a web search query, retrieve relevant passages that answer the query"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class DuRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="DuRetrieval",
        description="A Large-scale Chinese Benchmark for Passage Retrieval from Web Search Engine",
        reference="https://aclanthology.org/2022.emnlp-main.357.pdf",
        dataset={
            "path": "C-MTEB/DuRetrieval",
            "revision": "a1a333e290fe30b10f3f56498e3a0d911a693ced",
            "qrel_revision": "497b7bd1bbb25cb3757ff34d95a8be50a3de2279",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{qiu2022dureaderretrieval,
      title={DuReader_retrieval: A Large-scale Chinese Benchmark for Passage Retrieval from Web Search Engine},
      author={Yifu Qiu and Hongyu Li and Yingqi Qu and Ying Chen and Qiaoqiao She and Jing Liu and Hua Wu and Haifeng Wang},
      year={2022},
      eprint={2203.10232},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        prompt={
            "query": "Given a Chinese search query, retrieve web passages that answer the question"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class CovidRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CovidRetrieval",
        description="COVID-19 news articles",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/CovidRetrieval",
            "revision": "1271c7809071a13532e05f25fb53511ffce77117",
            "qrel_revision": "a9f41b7cdf24785531d12417ce0d1157ed4b39ca",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        prompt={
            "query": "Given a question on COVID-19, retrieve news articles that answer the question"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class CmedqaRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CmedqaRetrieval",
        description="Online medical consultation text. Used the CMedQAv2 as its underlying dataset.",
        reference="https://aclanthology.org/2022.emnlp-main.357.pdf",
        dataset={
            "path": "C-MTEB/CmedqaRetrieval",
            "revision": "cd540c506dae1cf9e9a59c3e06f42030d54e7301",
            "qrel_revision": "279d737f36c731c8ff6e2b055f31fe02216fa23d",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=["Medical", "Written"],
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        prompt={
            "query": "Given a Chinese community medical question, retrieve replies that best answer the question"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class EcomRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="EcomRetrieval",
        description="EcomRetrieval",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/EcomRetrieval",
            "revision": "687de13dc7294d6fd9be10c6945f9e8fec8166b9",
            "qrel_revision": "39c90699b034ec22ac45b3abf5b0bbb5ffd421f9",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        prompt={
            "query": "Given a user query from an e-commerce website, retrieve description sentences of relevant products"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class MedicalRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MedicalRetrieval",
        description="MedicalRetrieval",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/MedicalRetrieval",
            "revision": "2039188fb5800a9803ba5048df7b76e6fb151fc6",
            "qrel_revision": "37b8efec53c54c3d9c6af212f6710b62ccdf895c",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        prompt={
            "query": "Given a medical question, retrieve user replies that best answer the question"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True


class VideoRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="VideoRetrieval",
        description="VideoRetrieval",
        reference="https://arxiv.org/abs/2203.03367",
        dataset={
            "path": "C-MTEB/VideoRetrieval",
            "revision": "58c2597a5943a2ba48f4668c3b90d796283c5639",
            "qrel_revision": "faa71382b6a29cf1778d1f436b963e75cb5b927c",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["cmn-Hans"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation=None,
        prompt={
            "query": "Given a video search query, retrieve the titles of relevant videos"
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = load_retrieval_data(
            self.metadata_dict["dataset"]["path"],
            self.metadata_dict["dataset"]["revision"],
            self.metadata_dict["dataset"]["qrel_revision"],
            self.metadata_dict["eval_splits"],
        )
        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskZeroShotClassification import (
    AbsTaskZeroShotClassification,
)
from mteb.abstasks.TaskMetadata import TaskMetadata


class Caltech101ZeroShotClassification(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="Caltech101ZeroShot",
        description="Classifying images of 101 widely varied objects.",
        reference="https://ieeexplore.ieee.org/document/1384978",
        dataset={
            "path": "HuggingFaceM4/Caltech-101",
            "name": "with_background_category",
            "revision": "851374102055782c84f89b1b4e9d128a6568847b",
            "trust_remote_code": True,
        },
        type="ZeroShotClassification",
        category="i2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2003-01-01",
            "2004-01-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="created",
        bibtex_citation="""@INPROCEEDINGS{1384978,
        author={Li Fei-Fei and Fergus, R. and Perona, P.},
        booktitle={2004 Conference on Computer Vision and Pattern Recognition Workshop},
        title={Learning Generative Visual Models from Few Training Examples: An Incremental Bayesian Approach Tested on 101 Object Categories},
        year={2004},
        volume={},
        number={},
        pages={178-178},
        keywords={Bayesian methods;Testing;Humans;Maximum likelihood estimation;Assembly;Shape;Machine vision;Image recognition;Parameter estimation;Image databases},
        doi={10.1109/CVPR.2004.383}}""",
        descriptive_stats={
            "n_samples": {"test": 6084},
            "avg_character_length": {"test": 431.4},
        },
    )

    def get_candidate_labels(self) -> list[str]:
        return [
            f"a photo of a {name}."
            for name in self.dataset["test"].features[self.label_column_name].names
        ]



from __future__ import annotations

from mteb.abstasks.Image.AbsTaskImageClassification import AbsTaskImageClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class UCF101Classification(AbsTaskImageClassification):
    metadata = TaskMetadata(
        name="UCF101",
        description="""UCF101 is an action recognition data set of realistic
action videos collected from YouTube, having 101 action categories. This
version of the dataset does not contain images but images saved frame by
frame. Train and test splits are generated based on the authors' first
version train/test list.""",
        reference="https://huggingface.co/datasets/flwrlabs/ucf101",
        dataset={
            "path": "flwrlabs/ucf101",
            "revision": "1098eed48f2929443f47c39f3b5c814e16369c11",
        },
        type="ImageClassification",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2012-01-01",
            "2012-12-01",
        ),  # Estimated range for the collection of reviews
        domains=["Scene"],
        task_subtypes=["Activity recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation="""@misc{soomro2012ucf101dataset101human,
        title={UCF101: A Dataset of 101 Human Actions Classes From Videos in The Wild},
        author={Khurram Soomro and Amir Roshan Zamir and Mubarak Shah},
        year={2012},
        eprint={1212.0402},
        archivePrefix={arXiv},
        primaryClass={cs.CV},
        url={https://arxiv.org/abs/1212.0402},
        }""",
        descriptive_stats={
            "n_samples": {"test": 697222},
            "avg_character_length": {"test": 0},
        },
    )



from __future__ import annotations

from datasets import Dataset, DatasetDict, load_dataset

from mteb.abstasks.Image.AbsTaskAny2AnyRetrieval import AbsTaskAny2AnyRetrieval
from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGUAGES = {
    "ar": ["ara-Arab"],
    "bn": ["ben-Beng"],
    "cs": ["ces-Latn"],
    "da": ["dan-Latn"],
    "de": ["deu-Latn"],
    "el": ["ell-Grek"],
    "en": ["eng-Latn"],
    "es": ["spa-Latn"],
    "fa": ["fas-Arab"],
    "fi": ["fin-Latn"],
    "fil": ["fil-Latn"],
    "fr": ["fra-Latn"],
    "he": ["heb-Hebr"],
    "hi": ["hin-Deva"],
    "hr": ["hrv-Latn"],
    "hu": ["hun-Latn"],
    "id": ["ind-Latn"],
    "it": ["ita-Latn"],
    "ja": ["jpn-Jpan"],
    "ko": ["kor-Hang"],
    "mi": ["mri-Latn"],
    "nl": ["nld-Latn"],
    "no": ["nor-Latn"],
    "pl": ["pol-Latn"],
    "pt": ["por-Latn"],
    "quz": ["quz-Latn"],
    "ro": ["ron-Latn"],
    "ru": ["rus-Cyrl"],
    "sv": ["swe-Latn"],
    "sw": ["swa-Latn"],
    "te": ["tel-Telu"],
    "th": ["tha-Thai"],
    "tr": ["tur-Latn"],
    "uk": ["ukr-Cyrl"],
    "vi": ["vie-Latn"],
    "zh": ["zho-Hans"],
}


def _load_xm3600_data(
    path: str, langs: list, splits: str, cache_dir: str = None, revision: str = None
):
    corpus = {lang: {split: None for split in splits} for lang in langs}
    queries = {lang: {split: None for split in splits} for lang in langs}
    relevant_docs = {lang: {split: None for split in splits} for lang in langs}

    split = "test"

    for lang in langs:
        lang_data = load_dataset(
            path,
            split=lang,
            cache_dir=cache_dir,
            revision=revision,
            # trust_remote_code=True,
        )
        lang_corpus = lang_data.map(
            lambda x: {
                "id": "corpus-" + x["image_id"],
                "text": None,
                "modality": "image",
                "image": x["image"]["bytes"],
            },
            remove_columns=[
                "captions",
                "captions_tokenized",
                "captions_tokenized_lowercase",
                "image_locale",
                "image_id",
            ],
        )

        corpus[lang][split] = lang_corpus

        lang_data = lang_data.remove_columns(["image"])

        queries[lang][split] = []
        relevant_docs[lang][split] = {}

        for row in lang_data:
            image_id = "corpus-" + row["image_id"]
            for idx, caption in enumerate(row["captions"]):
                query_id = f"query-{row['image_id']}-{idx}"
                queries[lang][split].append(
                    {
                        "id": query_id,
                        "text": caption,
                        "modality": "text",
                        "image": None,
                    }
                )
                if query_id not in relevant_docs[lang][split]:
                    relevant_docs[lang][split][query_id] = {}
                relevant_docs[lang][split][query_id][image_id] = 1

        queries[lang][split] = Dataset.from_dict(
            {
                "id": [query["id"] for query in queries[lang][split]],
                "text": [query["text"] for query in queries[lang][split]],
                "modality": [query["modality"] for query in queries[lang][split]],
                "image": [None for _ in queries[lang][split]],
            }
        )
    corpus = DatasetDict({lang: DatasetDict(splits) for lang, splits in corpus.items()})
    queries = DatasetDict(
        {lang: DatasetDict(splits) for lang, splits in queries.items()}
    )
    relevant_docs = DatasetDict(relevant_docs)

    return corpus, queries, relevant_docs


class XM3600T2IRetrieval(MultilingualTask, AbsTaskAny2AnyRetrieval):
    metadata = TaskMetadata(
        name="XM3600T2IRetrieval",
        description="Retrieve images based on multilingual descriptions.",
        reference="https://aclanthology.org/2022.emnlp-main.45/",
        dataset={
            "path": "floschne/xm3600",
            "revision": "8d3e5665526c55a5855cd6ddfbaba2032bc7cee4",
            # "trust_remote_code": True,
        },
        type="Any2AnyMultilingualRetrieval",
        category="t2i",
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="ndcg_at_10",
        date=("2022-01-01", "2022-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Image Text Retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation="""@inproceedings{thapliyal2022crossmodal,
  title={Crossmodal-3600: A Massively Multilingual Multimodal Evaluation Dataset},
  author={Thapliyal, Ashish V and Tuset, Jordi Pont and Chen, Xi and Soricut, Radu},
  booktitle={Proceedings of the 2022 Conference on Empirical Methods in Natural Language Processing},
  pages={715--729},
  year={2022}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "ar": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "bn": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "cs": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "da": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "de": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "el": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "en": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "es": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "fa": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "fi": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "fil": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "fr": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "he": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "hi": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "hr": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "hu": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "id": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "it": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "ja": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "ko": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "mi": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "nl": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "pl": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "pt": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "quz": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "ro": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "ru": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "sv": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "sw": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "te": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "th": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "tr": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "uk": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "vi": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                    "zh": {
                        "average_document_length": 0.0,
                        "average_query_length": 0.0,
                        "num_documents": 7200,
                        "num_queries": 3600,
                        "average_relevant_docs_per_query": 2.0,
                    },
                }
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_xm3600_data(
            path=self.metadata_dict["dataset"]["path"],
            langs=self.hf_subsets,
            splits=self.metadata_dict["eval_splits"],
            cache_dir=kwargs.get("cache_dir", None),
            revision=self.metadata_dict["dataset"]["revision"],
        )

        self.data_loaded = True
